
%
% analyses the potential of each ortho mode, given a proper
% parameter tuple (x)
%
%  - re2_well_specs(idx, 1) <- well distance;
%  - re2_well_specs(idx, 2) <- well energy;
%  - re2_well_specs(idx, 3) <- well width;
%  - re2_well_specs(idx, 4) <- soft contact distance;
%  - re2_well_specs(idx, 5) <- 2nd derivative of energy at well;
%

function [re2_well_specs] = ortho_re2_well_analyser(ptup);

eps = 10^-3;
step = 1.0;
max_rad = 50;

[E0, sigma_c, a1, b1, c1, a2, b2, c2, ea1, eb1, ec1, ea2, eb2, ec2] = ...
    x_to_vars(ptup);

% well sepcs of RE^2
re2_well_specs = zeros(18, 5);

[N_modes, q_rot, q_vec] = ortho_mode_setup;

options = optimset('Display', 'off', 'MaxFunEvals', 50000, ...
    'MaxIter', 1000, 'TolFun', 1.0e-14 , 'TolX', 1.0e-14);

r_hat = zeros(3,1);
cA2 = zeros(3, 3);

for idx=1:N_modes
    
    r_hat(:) = q_vec(idx, :);
    cA2(:, :) = q_rot(idx, :, :);

    % well
    
    % finding lower limit
    min_rad = max_rad;
    prev_dU = 1;
    while true
        dU = get_dir_re2_int_energy(min_rad, r_hat, 0, cA2, E0, sigma_c, a1, b1, c1, ...
            ea1, eb1, ec1, a2, b2, c2, ea2, eb2, ec2, idx);
        if dU>prev_dU 
            break; 
        else, 
            prev_dU = dU;
            min_rad = min_rad - step; 
        end;
    end;

    [well_rad, well_depth] = fminbnd(@get_dir_re2_int_energy, min_rad, min_rad+2*step, ...
        options, r_hat, 0, cA2, E0, sigma_c, a1, b1, c1, ea1, eb1, ec1, a2, b2, c2, ea2, ...
        eb2, ec2, idx);

    % soft contact rad
    ddU = (get_dir_re2_int_energy(well_rad + eps, r_hat, 0, cA2, E0, sigma_c, a1, b1, c1, ...
            ea1, eb1, ec1, a2, b2, c2, ea2, eb2, ec2, idx) + get_dir_re2_int_energy(well_rad - eps, ...
            r_hat, 0, cA2, E0, sigma_c, a1, b1, c1, ea1, eb1, ec1, a2, b2, c2, ea2, eb2, ec2, idx) - ...
            2*well_depth)/eps^2;
        
    well_half_width = sqrt(-2*well_depth/ddU);
    if abs(well_half_width)>2, well_half_width = 1;, end;

    soft_contact_rad = fzero(@get_dir_re2_int_energy, well_rad - 0.5*well_half_width, options, r_hat, 0, ...
        cA2, E0, sigma_c, a1, b1, c1, ea1, eb1, ec1, a2, b2, c2, ea2, eb2, ec2, idx);

    % let us find well width now
    half_depth = well_depth/2;
        
    right_rad = fzero(@get_dir_re2_int_energy, well_rad + 1.5*well_half_width, options, r_hat, half_depth, ...
        cA2, E0, sigma_c, a1, b1, c1, ea1, eb1, ec1, a2, b2, c2, ea2, eb2, ec2, idx);
    left_rad = fzero(@get_dir_re2_int_energy, well_rad - 0.5*well_half_width, options, r_hat, half_depth, ...
        cA2, E0, sigma_c, a1, b1, c1, ea1, eb1, ec1, a2, b2, c2, ea2, eb2, ec2, idx);

    re2_well_specs(idx, 1) = well_rad;
    re2_well_specs(idx, 2) = well_depth;
    re2_well_specs(idx, 3) = right_rad-left_rad;
    re2_well_specs(idx, 4) = soft_contact_rad;
    re2_well_specs(idx, 5) = ddU;
    
end;
