
%
% orthogonal orientation sample generator
%
% changelog:
%
% - new method: - for each ortho mode:
%   (1) start sampling from gap
%   (2) stop sampling once the energy starts to raise
%

function [Nsamples, A2, r2, smp_energy, mode_index, bnds] = ...
  ortho_sample_generator_new(a_xyz_path, a_key_path, b_xyz_path, b_key_path, ...
    temp_xyz_path, temp_key_path, tinker_analyzer_path);
 
% consts
max_gap = 6; % maximum gap
step_size = 0.1; % step size toward the molecule at rest
max_energy = 5;  
max_sampling_after_min = 100;
beta = 0.5;
r_c = 10^12; % cutoff

% load original molecules
[a_Natoms, a_name, a_types, a_xyz, a_atomclass, a_n12] = load_xyz(a_xyz_path);
[b_Natoms, b_name, b_types, b_xyz, b_atomclass, b_n12] = load_xyz(b_xyz_path);

% let us find the self VdW energies first
a_self_energy = get_tinker_self_energy(a_xyz_path, a_key_path, tinker_analyzer_path);
b_self_energy = get_tinker_self_energy(b_xyz_path, b_key_path, tinker_analyzer_path);

% analyzing the bounding box of the given molecule
a_ux = max(a_xyz(:,1));
a_lx = -min(a_xyz(:,1));
a_uy = max(a_xyz(:,2));
a_ly = -min(a_xyz(:,2));
a_uz = max(a_xyz(:,3));
a_lz = -min(a_xyz(:,3));

b_ux = max(b_xyz(:,1));
b_lx = -min(b_xyz(:,1));
b_uy = max(b_xyz(:,2));
b_ly = -min(b_xyz(:,2));
b_uz = max(b_xyz(:,3));
b_lz = -min(b_xyz(:,3));

% queue setup
[N_modes, q_rot, q_vec] = ortho_mode_setup;
q_min_r = zeros(N_modes, 1);

q_min_r(1) = a_ux + b_lx;
q_min_r(2) = a_uy + b_ly;
q_min_r(3) = a_uz + b_lz;
q_min_r(4) = a_ux + b_uy;
q_min_r(5) = a_uy + b_lx;
q_min_r(6) = a_uz + b_lz;
q_min_r(7) = a_ux + b_lz;
q_min_r(8) = a_uy + b_ly;
q_min_r(9) = a_uz + b_ux;
q_min_r(10) = a_ux + b_lz;
q_min_r(11) = a_uy + b_lx;
q_min_r(12) = a_uz + b_ly;
q_min_r(13) = a_ux + b_ux;
q_min_r(14) = a_uy + b_lz;
q_min_r(15) = a_uz + b_ly;
q_min_r(16) = a_ux + b_ly;
q_min_r(17) = a_uy + b_lz;
q_min_r(18) = a_uz + b_lx;

bnds = zeros(6, 1);
bnds(1) = (a_ux + a_lx)/2;
bnds(2) = (a_uy + a_ly)/2;
bnds(3) = (a_uz + a_lz)/2;
bnds(4) = (b_ux + b_lx)/2;
bnds(5) = (b_uy + b_ly)/2;
bnds(6) = (b_uz + b_lz)/2;

% init
Nsamples = 18; % minimum samples
A2 = zeros(Nsamples, 3, 3);
r2 = zeros(Nsamples, 3);
smp_energy = zeros(Nsamples, 1);
weight = zeros(Nsamples, 1);
mode_index = zeros(N_modes, 2);

% sample generation

curr_A2 = zeros(3, 3);
curr_vec = zeros(3, 1);

ctr = 1;

for iter_mode=1:N_modes
    
    curr_max_r = q_min_r(iter_mode) + max_gap;
    curr_A2(:, :) = q_rot(iter_mode, :, :);
    curr_vec = q_vec(iter_mode, :)';
    
    curr_r = curr_max_r;
    
    inner_counter = 1;
    prev_energy = 0;
    after_min = 0;
    min_flag = 0;
    mode_index(iter_mode, 1) = ctr;
    
    while true
        
      r2(ctr, :) = curr_r.*curr_vec';
      A2(ctr, :, :) = curr_A2(:, :);
      
      smp_energy(ctr) = get_tinker_int_energy(temp_xyz_path, temp_key_path, ...
            tinker_analyzer_path, a_Natoms, a_name, a_types, a_xyz, a_atomclass, a_n12, ...
            b_Natoms, b_name, b_types, b_xyz, b_atomclass, b_n12, r2(ctr, :), curr_A2, ...
            a_self_energy, b_self_energy);
      
%    smp_energy(ctr) = get_mm3_int_energy(r2(ctr, :), curr_A2, a_Natoms, a_atomclass, a_xyz, ...
%        b_Natoms, b_atomclass, b_xyz, r_c);

    if smp_energy(ctr)>prev_energy, min_flag=1;, end;
      
      if min_flag==1, after_min = after_min+1;, end;
      
      if (after_min>max_sampling_after_min || smp_energy(ctr)>max_energy)
          mode_index(iter_mode, 2) = ctr - 1;
          break; 
      end;
      
      disp(['   mode idx: ' num2str(iter_mode) '/' num2str(N_modes) ...
            '  #' num2str(inner_counter) ...
            '  U_int: ' num2str(smp_energy(ctr))]);
  
      prev_energy = smp_energy(ctr);
      ctr = ctr + 1;
      inner_counter = inner_counter + 1;
      curr_r = curr_r - step_size;
           
  end;
    
end;

% output
Nsamples = ctr - 1;
smp_energy = smp_energy(1:Nsamples);
r2 = r2(1:Nsamples, :);
A2 = A2(1:Nsamples, :, :);
