
%
% analyses the potential of each ortho mode, given a proper orthogonal
% sampling (Nsamples, A2, r2, smp_energy, mode_index)
%
%  - well_specs(idx, 1) <- well distance;
%  - well_specs(idx, 2) <- well energy;
%  - well_specs(idx, 3) <- well width;
%  - well_specs(idx, 4) <- soft contact distance;
%  - well_specs(idx, 5) <- 2nd derivative of energy at well;
%

function [well_specs] = ortho_well_analyser(Nsamples, A2, r2, smp_energy, mode_index);

smoothing_flag = 0; % apply Savitzky-Golay smoothing to data? (used for PMF)

well_specs = zeros(18, 5);

options = optimset('Display', 'off', 'MaxFunEvals', 50000, ...
    'MaxIter', 1000, 'TolFun', 1.0e-10 , 'TolX', 1.0e-10, ...
    'MaxPCGIter', 20);

for idx=1:18

    lower_idx = mode_index(idx, 1);
    upper_idx = mode_index(idx, 2);
    num = upper_idx-lower_idx+1;
    r_norm = zeros(num, 1);
    cA2 = zeros(3, 3);
    cr2 = zeros(3, 1);

    for iter=1:num, r_norm(iter) = norm(r2(iter-1+lower_idx,:));, end;

    max_r = max(r_norm);
    min_r = min(r_norm);
    x0 = (min_r+max_r)/2;
    
    if smoothing_flag==1
        % Savitzky-Golay smoothing filter
        smooth_energy = sgolayfilt(smp_energy(lower_idx:upper_idx), 4, 9);
    else
        smooth_energy = smp_energy(lower_idx:upper_idx);
    end;
    
    % PCHIP fit
    pp = pchip(r_norm, smooth_energy);
    
    [well_rad, well_depth] = fminbnd(@ppval_wrapper, min_r, max_r, options, pp, 0);

    well_specs(idx, 1) = well_rad;
    well_specs(idx, 2) = well_depth;
    
    % soft contact radius
    soft_contact_rad = fzero(@ppval_wrapper, (well_rad + min_r)/2, options, pp, 0);
    well_specs(idx, 4) = soft_contact_rad;
    
    % let us find well width now
    half_depth = well_depth/2;
    right_rad = fzero(@ppval_wrapper, 2*well_rad - min_r, options, pp, half_depth);
    left_rad = fzero(@ppval_wrapper, (well_rad + min_r)/2, options, pp, half_depth);
    well_specs(idx, 3) = right_rad - left_rad;

    % 2nd diff. at well
    eps = 10^-3;
    ddU = (ppval_wrapper(well_rad+eps,pp,0)+ppval_wrapper(well_rad-eps,pp,0)-...
        2*ppval_wrapper(well_rad,pp,0))/eps^2;
    well_specs(idx, 5) = ddU;
    
end;
